@@ -1,6 +1,6 @@
 # vim: set expandtab ts=4 sw=4 filetype=python:
 
-import warnings
+import contextlib, multiprocessing, os, subprocess, sys, time, warnings
 from UserDict import UserDict
 from copy import copy
 from datetime import datetime, timedelta
@@ -121,6 +121,7 @@ def daterange_generator(from_this_date, increment, until_this_date):
         yield x
         x += increment
 
+
 def deprecated(message='Deprecated!'):
     """
     Raise a DeprecationWarning before calling the function.
@@ -131,3 +132,68 @@ def deprecated(message='Deprecated!'):
         return f(*args, **kwargs)
 
     return decorator(_d)
+
+
+def walkup(path="right here"):
+
+    """
+    Yield paths closer and closer to the to of the filesystem.
+    """
+
+    if path == "right here":
+        path = os.getcwd()
+
+    at_top = False
+    while not at_top:
+        yield path
+        parent_path = os.path.dirname(path)
+        if parent_path == path:
+            at_top = True
+        else:
+            path = parent_path 
+
+
+def send_through_pager(s):
+    """
+    Open up this user's pager and send string s through it.
+
+    If I don't find a $PAGER variable and "which less" fails, I'll just
+    print the string.
+    """
+
+    pager = os.environ.get('PAGER')
+
+    if not pager:
+        return_code = subprocess.call(['which', 'less'])
+
+        # The which program returns zero when it found stuff.
+        if return_code == 0:
+            pager = 'less'
+
+    if pager:
+        p = subprocess.Popen([pager], stdin=subprocess.PIPE)
+        p.communicate(s)
+
+    else:
+        print(s)
+
+
+def draw_ascii_spinner(delay=0.2):
+    for char in '/-\\|':
+        sys.stdout.write(char)
+        sys.stdout.flush()
+        time.sleep(delay)
+        sys.stdout.write('\r')
+
+
+def spin_forever():
+    while 1:
+        draw_ascii_spinner()
+
+
+@contextlib.contextmanager
+def spinning_distraction():
+    p = multiprocessing.Process(target=spin_forever)
+    yield
+    p.terminate()
+
